@@ -13,8 +13,9 @@ extern "C" {
 #define XAPIAN_FILE_PREFIX "xapian-indexes"
 #define XAPIAN_TERM_SIZELIMIT 245
 #define XAPIAN_COMMIT_LIMIT 1000
+#define XAPIAN_COMMIT_TIMEOUT 300
 #define XAPIAN_WILDCARD "wldcrd"
-#define XAPIAN_EXPUNGE_SIZE 4
+#define XAPIAN_EXPUNGE_SIZE 5
 
 #define HDRS_NB 9
 static const char * hdrs_emails[HDRS_NB] = { "uid", "subject", "from", "to",  "cc",  "bcc",  "messageid", "body", ""  };
@@ -29,24 +30,25 @@ struct xapian_fts_backend
 	long partial,full;
 	bool attachments;
 
-        struct mailbox *box;
 	char * guid;
+	char * boxname;
 	char * db;
 
-	char * oldbox;
-	char * oldbox_name;
+	char * old_guid;
+	char * old_boxname;
 
 	Xapian::Database * dbr;
-
 	Xapian::WritableDatabase * dbw;
-	long nb_updates;
+	sqlite3 * db_expunge;
 
-	sqlite3 * dbexpunge;
+	long commit_updates;
+	long commit_time;
 
 	long perf_pt;
 	long perf_nb;
 	long perf_uid;
 	long perf_dt;
+
 };
 
 struct xapian_fts_expunge
@@ -86,11 +88,12 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 	backend->dbw = NULL;
 	backend->dbr = NULL;
 	backend->db = NULL;
-	backend->box = NULL;
+	backend->guid = NULL;
 	backend->path = NULL;
-	backend->oldbox = NULL;
-	backend->oldbox_name = NULL;
+	backend->old_guid = NULL;
+	backend->old_boxname = NULL;
 	backend->attachments = false;
+	backend->db_expunge = NULL;
 	verbose = 0;
 	backend->partial = 0;
 	backend->full = 0;
@@ -178,12 +181,15 @@ static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 
 	fts_backend_xapian_unset_box(backend);
 
-	if(backend->oldbox != NULL) i_free(backend->oldbox);
-	backend->oldbox = NULL;
-	if(backend->oldbox_name != NULL) i_free(backend->oldbox_name);
-	backend->oldbox_name = NULL;
+	if(backend->old_guid != NULL) i_free(backend->old_guid);
+	backend->old_guid = NULL;
+
+	if(backend->old_boxname != NULL) i_free(backend->old_boxname);
+	backend->old_boxname = NULL;
+
 	if(backend->path != NULL) i_free(backend->path);
 	backend->path = NULL;
+
 	i_free(backend);
 }
 
@@ -213,12 +219,12 @@ static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct
 	}
 	catch(Xapian::Error e)
 	{
-		i_error("FTS Xapian: fts_backend_xapian_get_last_uid %s",backend->box->name);
+		i_error("FTS Xapian: fts_backend_xapian_get_last_uid for '%s' (%s)",backend->boxname,backend->guid);
 		i_error("FTS Xapian: %s",e.get_msg().c_str());
 		return -1;
 	}
 
-	if(verbose>0) i_info("FTS Xapian: Get last UID of %s (%s) = %d",backend->box->name,backend->guid,*last_uid_r);
+	if(verbose>0) i_info("FTS Xapian: Get last UID of %s (%s) = %d",backend->boxname,backend->guid,*last_uid_r);
 
         return 0;
 }
@@ -306,18 +312,18 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 	ctx->tbi_isfield=false;
         ctx->tbi_uid=0;
 
-	if(backend->box == NULL)
+	if(backend->guid == NULL)
 	{
 		if(verbose>0) i_warning("FTS Xapian: Build key %s with no mailbox",key->hdr_name);
 		return FALSE;
 	}
 
-	if((backend->oldbox == NULL) || (strcmp(backend->oldbox,backend->guid)!=0))
+	if((backend->old_guid == NULL) || (strcmp(backend->old_guid,backend->guid)!=0))
         {
 		fts_backend_xapian_oldbox(backend);
-		backend->oldbox = i_strdup(backend->guid);
-		backend->oldbox_name = i_strdup(backend->box->name);
-		if(verbose>0) i_info("FTS Xapian: Start indexing '%s' (%s)",backend->box->name,backend->guid);
+		backend->old_guid = i_strdup(backend->guid);
+		backend->old_boxname = i_strdup(backend->boxname);
+		if(verbose>0) i_info("FTS Xapian: Start indexing '%s' (%s)",backend->boxname,backend->guid);
 	}
 
 	/* Performance calculator*/
@@ -338,7 +344,7 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 			r=backend->perf_nb*1000.0;
 			r=r/dt;
 		}
-		if(verbose>0) i_info("FTS Xapian: Partial indexing '%s' (%ld msgs in %ld ms, rate: %.1f)",backend->box->name,backend->perf_nb,dt,r);
+		if(verbose>0) i_info("FTS Xapian: Partial indexing '%s' (%ld msgs in %ld ms, rate: %.1f)",backend->boxname,backend->perf_nb,dt,r);
 	}
 	/* End Performance calculator*/
 
@@ -425,21 +431,20 @@ static void fts_backend_xapian_update_unset_build_key(struct fts_backend_update_
 
 static int fts_backend_xapian_refresh(struct fts_backend * _backend)
 {
-        struct xapian_fts_backend *backend =
-                (struct xapian_fts_backend *) _backend;
+	struct xapian_fts_backend *backend =
+		(struct xapian_fts_backend *) _backend;
 
-	fts_backend_xapian_commit(backend,"refresh");
+	struct timeval tp;
+        gettimeofday(&tp, NULL);
+        long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+
+	fts_backend_xapian_release(backend,"refresh");
+	fts_backend_xapian_expunge(backend,"refresh");
+	fts_backend_xapian_commit(backend,"refresh", current_time);
         
-        if(backend->dbr !=NULL)
-        {
-		backend->dbr->close();
-                delete(backend->dbr);
-                backend->dbr = NULL;
-        }
-        return 0;
+	return 0;
 }
 
-
 static int fts_backend_xapian_update_build_more(struct fts_backend_update_context *_ctx, const unsigned char *data, size_t size)
 {
 	struct xapian_fts_backend_update_context *ctx =
@@ -472,17 +477,40 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 		ok=fts_backend_xapian_index_text(backend->dbw,ctx->tbi_uid,ctx->tbi_field, &d2, backend->partial,backend->full);
     	}
 
-	backend->nb_updates++;
-	if(backend->nb_updates>XAPIAN_COMMIT_LIMIT)
+	backend->commit_updates++;
+
+	struct timeval tp;
+	gettimeofday(&tp, NULL);
+	long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+
+	if( (backend->commit_updates>XAPIAN_COMMIT_LIMIT) || ((current_time - backend->commit_time) > XAPIAN_COMMIT_TIMEOUT*1000) )
 	{
-		if(verbose>1) i_info("FTS Xapian: Refreshing...");
-		fts_backend_xapian_refresh( ctx->ctx.backend);
+		if(verbose>1) i_info("FTS Xapian: Refreshing after %ld ms and %ld updates...", current_time - backend->commit_time, backend->commit_updates);
+		fts_backend_xapian_release(backend,"refreshing");
+		fts_backend_xapian_expunge(backend,"refreshing");
+		fts_backend_xapian_commit(backend,"refreshing", current_time);
 	}
     	
 	if(!ok) return -1;
 	return 0;
 }
 
+static int fts_backend_xapian_optimize(struct fts_backend *_backend)
+{
+	struct xapian_fts_backend *backend =
+		(struct xapian_fts_backend *) _backend;
+
+	if(verbose>0) i_info("Optimize function");
+	
+	struct timeval tp;
+        gettimeofday(&tp, NULL);
+        long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+	
+	fts_backend_xapian_expunge(backend,"optimize");
+        fts_backend_xapian_commit(backend,"optimize",current_time);
+
+	return 0;
+}
 
 static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 {
@@ -496,7 +524,7 @@ static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 		return -1;
 	}
 
-	return ftw(backend->path,fts_backend_xapian_empty_db,1024);
+	return ftw(backend->path,fts_backend_xapian_empty_db,512);
 }
 
 static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbox *box, struct mail_search_arg *args, enum fts_lookup_flags flags, struct fts_result *result)
@@ -507,7 +535,12 @@ static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbo
 	if(fts_backend_xapian_set_box(backend, box)<0)
 		return -1;
 
-	fts_backend_xapian_commit(backend,"lookup");
+	/* Performance calc */
+	struct timeval tp;
+        gettimeofday(&tp, NULL);
+        long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
+
+	fts_backend_xapian_commit(backend,"lookup", current_time);
 
 	if(!fts_backend_xapian_check_read(backend))
         {
@@ -515,11 +548,6 @@ static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbo
                 return -1;
         }
 
-	/* Performance calc */
-	struct timeval tp;
-        gettimeofday(&tp, NULL);
-        long dt = tp.tv_sec * 1000 + tp.tv_usec / 1000;
-
 	bool is_and=false;
 
 	if((flags & FTS_LOOKUP_FLAG_AND_ARGS) != 0)
@@ -560,9 +588,12 @@ static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbo
 	delete(qs);
 
 	/* Performance calc */
-        gettimeofday(&tp, NULL);
-        dt = tp.tv_sec * 1000 + tp.tv_usec / 1000 - dt;
-	if(verbose>0) i_info("FTS Xapian: %ld results in %ld ms",n,dt);
+        if(verbose>0)
+	{
+		gettimeofday(&tp, NULL);
+        	long dt = tp.tv_sec * 1000 + tp.tv_usec / 1000 - current_time;
+		i_info("FTS Xapian: %ld results in %ld ms",n,dt);
+	}
 
 	return 0;
 }
@@ -603,7 +634,7 @@ struct fts_backend fts_backend_xapian = {
 		fts_backend_xapian_update_build_more,
 		fts_backend_xapian_refresh,
 		fts_backend_xapian_rescan,
-		NULL,
+		fts_backend_xapian_optimize,
 		fts_backend_default_can_lookup,
 		fts_backend_xapian_lookup,
 		fts_backend_xapian_lookup_multi,
